
#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''Classes to uniformly query DiGIR or BioCASE or TAPIR services for a single record.
Its heavily based on libxml2.
'''


__version__ = "0.1"


############################################################################################################
#
#   IMPORTS
#
#===========================================================================================================
from mx.DateTime import now
import string
from types import *
import logging
import configuration
# xmlDOM.name : returns the node name
#	  * type : returns a string indicating the node type
#	  * content : returns the content of the node, it is based on xmlNodeGetContent() and hence is recursive.
#	  * parent , children, last, next, prev, doc, properties: pointing to the associated element in the tree, those may return None in case no such link exists.
import libxml2


import timeoutsocket


import urllib

# debugging to file
log = logging.getLogger("wrapper")


############################################################################################################
#
#   CLASSES
#
#===========================================================================================================
class ServiceError(Exception):
	def __init__(self, *args):
		Exception.__init__(self, *args)

class HttpError(ServiceError):
	def __init__(self, url):
		ServiceError.__init__(self, url)
		self.url=url
		
class MalformedResponseError(ServiceError):
	def __init__(self, url, response):
		ServiceError.__init__(self, url)
		self.response=response
		self.url = url

class UnknownSchemaError(Exception):
	def __init__(self, *args):
		Exception.__init__(self, *args)
		self.schema = args
		
class XMLError(ServiceError):
	def __init__(self, xml):
		ServiceError.__init__(self, xml)
		self.xml = xml
		
class NoResultError(ServiceError):
	def __init__(self, res):
		ServiceError.__init__(self, res)
		self.res = res
		
class ProtocolError(Exception):
	def __init__(self,*args):
		Exception.__init__(self, *args)
		self.tooShort = args	
		
class CatalogError(Exception):
	def __init__(self,*args):
		Exception.__init__(self, *args)
		self.catalog = args
# ----------------------------------------------------------------------------
class recordStatus:
	'''Class containing attribute information of element <content> like record state.'''
	def __init__(self):
		self.start = 0
		self.count = 0
		self.drop  = 0
		self.total = 0
		
	def __repr__(self):
		return "RecordStatus: start=%i, count=%i, drop=%i, total=%i"%(self.start, self.count, self.drop, self.total)

	def getNextPageIndex(self):
		'''Returns the index used for start of the next page or None if there is no more page'''
		if self.total > self.start+self.drop+self.count:
			return self.start+self.drop+self.count+1
		else:
			return None


# ----------------------------------------------------------------------------
class QueryGenerator:
	'''A class to generate protocol xml strings.'''
	
	NS_DiGIR   = "http://digir.net/schema/protocol/2003/1.0"
	NS_BioCASe = "http://www.biocase.org/schemas/protocol/1.3"
	NS_TAPIR   = "http://rs.tdwg.org/tapir/1.0"
	OTHER_CATALOG_TAG='CatalogNumberText'
	# known content schemas and their unit guid concepts:
	schema2conceptIDs = {
		'http://www.tdwg.org/schemas/abcd/2.06':[
			"/DataSets/DataSet/Units/Unit/UnitID",
			"/DataSets/DataSet/Units/Unit/SourceID",
			"/DataSets/DataSet/Units/Unit/SourceInstitutionID",
			"abcd2.06",
			],
		'http://www.tdwg.org/schemas/abcd/2.05':[
			"/DataSets/DataSet/Units/Unit/UnitID",
			"/DataSets/DataSet/Units/Unit/SourceID",
			"/DataSets/DataSet/Units/Unit/SourceInstitutionID",
			"abcd2.05",
			],
		'http://www.tdwg.org/schemas/abcd/1.2':[
			"/DataSets/DataSet/Units/Unit/UnitID",
			"/DataSets/DataSet/OriginalSource/SourceName",
			"/DataSets/DataSet/OriginalSource/SourceInstitutionCode",
			"abcd1.2",
			],
		'http://digir.net/schema/conceptual/darwin/2003/1.0':[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"darwincore",
			],
		'http://www.namespacetbd.org/darwin2':[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"darwincore",
			],
		 'http://www.namespaceTBD.org/darwin2' :[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"darwincore",
			],						 
		 'http://www.iobis.org/obis':[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"obis",
			]  ,
		 'http://digir.net/schema/conceptual/darwin/manis/1.21' :[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"manis",
			]	,
		'http://digir.net/schema/protocol/2003/1.0':[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"darwincore",
			],
			'http://manis.digir.net/schema/conceptual/darwin/2003/1.0' :[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"manis",
			],
		'http://rs.tdwg.org/dwc/dwcore'	:[
			"CatalogNumber",
			"CollectionCode",
			"InstitutionCode",
			"darwincore",
			],
		'http://rs.tdwg.org/tapir/1.0':[
			"http://rs.tdwg.org/dwc/dwcore/CatalogNumber",
			"http://rs.tdwg.org/dwc/dwcore/CollectionCode",
			"http://rs.tdwg.org/dwc/dwcore/InstitutionCode",
			"darwincore",
		]															
	 }
	schema2outputmodel = {
	 	'http://rs.tdwg.org/dwc/dwcore':[
			"http://rs.tdwg.org/dwc/dwcore/CatalogNumber",
			"http://rs.tdwg.org/dwc/dwcore/CollectionCode",
			"http://rs.tdwg.org/dwc/dwcore/InstitutionCode",
			"darwincore",
			"http://rs.tdwg.org/tapir/cs/dwc/1.4/model/dw_core_geo_cur.xml",
		],
		'http://digir.net/schema/conceptual/darwin/2003/1.0':[
			"http://digir.net/schema/conceptual/darwin/2003/1.0/CatalogNumber",
			"http://digir.net/schema/conceptual/darwin/2003/1.0/CollectionCode",
			"http://digir.net/schema/conceptual/darwin/2003/1.0/InstitutionCode",
			"darwincore",
			"http://rs.tdwg.org/tapir/cs/dwc/1.4/model/dw_core_geo_cur.xml"
		],
		'http://rs.tdwg.org/dwc/dwcore/':[
             "http://rs.tdwg.org/dwc/dwcore/CatalogNumber",
             "http://rs.tdwg.org/dwc/dwcore/CollectionCode",
             "http://rs.tdwg.org/dwc/dwcore/InstitutionCode",
             "darwincore",
             "http://rs.tdwg.org/tapir/cs/dwc/1.4/model/dw_core_geo_cur.xml",
                ],
		'amettreajour':[
			"http://digir.net/schema/conceptual/darwin/2003/1.0/CatalogNumber",
			"http://digir.net/schema/conceptual/darwin/2003/1.0/CollectionCode",
			"http://digir.net/schema/conceptual/darwin/2003/1.0/InstitutionCode",
			"abcd2.06",
			"http://rs.tdwg.org/tapir/cs/dwc/1.4/model/abcd206.xml"
		]
	 }
	# ------------------------------------------------------------------------------------
	def __init__(self, protocolNS=None):
		if protocolNS is None:
			protocolNS = QueryGenerator.NS_BioCASe
		elif protocolNS.lower() == 'biocase':
			protocolNS = QueryGenerator.NS_BioCASe
		elif protocolNS.lower() == 'digir':
			protocolNS = QueryGenerator.NS_DiGIR			
		elif protocolNS.lower() == 'tapir':
			protocolNS = QueryGenerator.NS_TAPIR
		self._type = protocolNS
#		print "protocoleNS :%s" %self._type
		
	# ----------------------------------------------------------------------------
	def _getOperatorString(self, conceptID, value, operator='equals'):
		if self._type == QueryGenerator.NS_BioCASe:
			return '<%s path="%s">%s</%s>'%(operator, conceptID, escapeXML(value), operator )
		elif self._type== QueryGenerator.NS_DiGIR:
			return '<%s> <darwin:%s>%s</darwin:%s> </%s>'%(operator, conceptID, escapeXML(value), conceptID, operator )
		elif self._type== QueryGenerator.NS_TAPIR:
			return '<%s><concept id="%s"/><literal value="%s"/></%s>'%(operator, conceptID, escapeXML(value), operator )
	# ----------------------------------------------------------------------------
	def _logicallyCombineOperators(self, op1, op2, LOP='and'):
		return '<%s> %s %s </%s>'%(LOP, op1, op2, LOP)
	
	# ------------------------------------------------------------------------------------
	def getUnitProtocol(self, contentSchemaNamespace, serviceURL, unitID, collID=None, instID=None,otherCatalogTag=False ):
		'''
		serviceURL: service access point URL. In case of DiGIR services the resource should be prefixed as RESOURCE@ACCESSPOINT
		contentSchemaNamespace: the namespace of the content/conceptual schema
		In the case of DiGIR resources should be provided in the form of  as the destination string.'''
		# if digir service_url parse resource of form RESOURCE@ACCESSPOINT
		
		if serviceURL is not None:
			if serviceURL.count("@") == 1:
				digirResource, destination = serviceURL.split("@")
			else:
				digirResource = ''
				destination   = serviceURL
		else:
			digirResource = ''
			destination   = ''
		if self._type == QueryGenerator.NS_TAPIR:
			if serviceURL[len(serviceURL)-1] !="/":
				serviceURL=serviceURL+"/"
			if destination[len(destination)-1] !="/":
				destination=destination+"/"
			clientCap=ClientCapabilities(destination,serviceURL)
			contentSchemaNamespace=clientCap.getCapabilities()
			outputmodel=clientCap.getOutputmodel(contentSchemaNamespace)
			if contentSchemaNamespace is None:
				raise UnknownSchemaError("no namespace")
			if not QueryGenerator.schema2outputmodel.has_key(contentSchemaNamespace.lower()):
				raise UnknownSchemaError(contentSchemaNamespace)
			conceptIDs = QueryGenerator.schema2outputmodel[contentSchemaNamespace.lower()]		
			
		else:
			# get concept IDs by hardcoded content schema knowledge
			if not QueryGenerator.schema2conceptIDs.has_key(contentSchemaNamespace.lower()):
				raise UnknownSchemaError(contentSchemaNamespace)
			conceptIDs = QueryGenerator.schema2conceptIDs[contentSchemaNamespace.lower()]
			
		#tag is different for manis provider
		if otherCatalogTag:
			catalogNumberTag = QueryGenerator.OTHER_CATALOG_TAG
		else:
			catalogNumberTag = conceptIDs[0] 
			
		# prepare filter string
		filter = self._getOperatorString(conceptID=catalogNumberTag, value=unitID)

		if collID is not None:
			umlaut=checkUmlaut(collID)
			if umlaut is not None:
				op2 = self._getOperatorString(conceptID=conceptIDs[1], value=umlaut, operator='like')
			else:
				op2 = self._getOperatorString(conceptID=conceptIDs[1], value=collID)
			filter = self._logicallyCombineOperators(filter, op2, 'and')
		if instID is not None:
			umlaut=checkUmlaut(instID)
			if umlaut is not None:
				op2 = self._getOperatorString(conceptID=conceptIDs[2], value=umlaut, operator='like')
			else:
				op2 = self._getOperatorString(conceptID=conceptIDs[2], value=instID)
			filter = self._logicallyCombineOperators(filter, op2, 'and')
		# other things
		start = 0
		limit = 10
		count = False
		# build protocol instance string
		if self._type == QueryGenerator.NS_BioCASe:
			return u'''<?xml version="1.0" encoding="UTF-8"?>
		<!--XML generated by BioCASE query tool for a single database, written by Markus Doering and Javier de la Torre, Botanic Garden and Botanical Museum, Berlin.-->
		<request 
			xmlns="%s" 
			xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
			xsi:schemaLocation="%s http://www.bgbm.org/biodivinf/Schema/protocol_1_3.xsd">
			<header>
				<version software="querytool">%s</version>
				<sendTime>%s</sendTime>
				<source/>
				<destination>%s</destination>
				<type>search</type>
			</header>
			<search>
				<requestFormat>%s</requestFormat>
				<responseFormat start="%i" limit="%i">%s</responseFormat>
				<filter>
				  %s
				</filter>
				<count>%s</count>
			</search>
		</request>
		''' %(self._type, self._type, __version__, str(now()), destination, contentSchemaNamespace, start, limit, contentSchemaNamespace, filter, str(count))
	
		elif self._type == QueryGenerator.NS_DiGIR :
			return  u'''<?xml version="1.0" encoding="UTF-8"?>
		<request xmlns='http://digir.net/schema/protocol/2003/1.0'
	xmlns:xsd='http://www.w3.org/2001/XMLSchema'
	xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
	xmlns:digir='http://digir.net/schema/protocol/2003/1.0'
	xmlns:darwin='http://digir.net/schema/conceptual/darwin/2003/1.0'
	xmlns:dwc='http://digir.net/schema/conceptual/darwin/2003/1.0'
	xsi:schemaLocation='http://digir.net/schema/protocol/2003/1.0 http://digir.sourceforge.net/schema/protocol/2003/1.0/digir.xsd http://digir.net/schema/conceptual/darwin/2003/1.0 http://digir.sourceforge.net/schema/conceptual/darwin/2003/1.0/darwin2.xsd'>
 
        
		<header>
		<version>1.0.0</version>
		<sendTime>%s</sendTime>
		<source/>
		<destination resource="%s">%s</destination>
		<type>search</type>
		</header>
		<search>
		<filter>%s</filter>
		<records  start="%i" limit="%i">
		<structure schemaLocation="http://search.biocase.org/europe/static/darwinObis.xsd"/>
		</records>
		<count>false</count>
		</search>
		</request>'''%( str(now()), digirResource, destination, filter,start, limit)
#			return u'''<?xml version="1.0" encoding="UTF-8"?>
#<request 
#	xmlns="%s" 
#	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
#	xmlns:digir="http://digir.net/schema/protocol/2003/1.0"
#	xmlns:darwin="http://digir.net/schema/conceptual/darwin/2003/1.0"
#	xmlns:obis="http://www.iobis.org/obis"
#	xmlns:manis="http://digir.net/schema/protocol/2003/1.0"
#	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" >
#	<header>
#	  <version>1.0</version>
#	  <sendTime>%s</sendTime>
#	  <source/>
#	  <destination resource="%s">%s</destination>
#	  <type>search</type>
#	</header>
#	<search>
#	  <filter>
#		  %s
#	  </filter>
#	  <records start="%i" limit="%i">
#		<structure>
#		  <xsd:element name="record">
#			<xsd:complexType>
#			  <xsd:sequence>
#				<xsd:element ref="darwin:DateLastModified"/>
#				<xsd:element ref="darwin:InstitutionCode"/>
#				<xsd:element ref="darwin:CollectionCode"/>
#				<xsd:element ref="darwin:%s"/>
#				<xsd:element ref="darwin:ScientificName"/>
#				<xsd:element ref="darwin:BasisOfRecord"/>
#				<xsd:element ref="darwin:Kingdom"/>
#				<xsd:element ref="darwin:Phylum"/>
#				<xsd:element ref="darwin:Class"/>
#				<xsd:element ref="darwin:Order"/>
#				<xsd:element ref="darwin:Family"/>
#				<xsd:element ref="darwin:Genus"/>
#				<xsd:element ref="darwin:Species"/>
#				<xsd:element ref="darwin:Subspecies"/>
#				<xsd:element ref="darwin:ScientificNameAuthor"/>
#				<xsd:element ref="darwin:IdentifiedBy"/>
#				<xsd:element ref="darwin:YearIdentified"/>
#				<xsd:element ref="darwin:MonthIdentified"/>
#				<xsd:element ref="darwin:DayIdentified"/>
#				<xsd:element ref="darwin:TypeStatus"/>
#				<xsd:element ref="darwin:CollectorNumber"/>
#				<xsd:element ref="darwin:FieldNumber"/>
#				<xsd:element ref="darwin:Collector"/>
#				<xsd:element ref="darwin:YearCollected"/>
#				<xsd:element ref="darwin:MonthCollected"/>
#				<xsd:element ref="darwin:DayCollected"/>
#				<xsd:element ref="darwin:JulianDay"/>
#				<xsd:element ref="darwin:TimeOfDay"/>
#				<xsd:element ref="darwin:ContinentOcean"/>
#				<xsd:element ref="darwin:Country"/>
#				<xsd:element ref="darwin:StateProvince"/>
#				<xsd:element ref="darwin:County"/>
#				<xsd:element ref="darwin:Locality"/>
#				<xsd:element ref="darwin:Longitude"/>
#				<xsd:element ref="darwin:Latitude"/>
#				<xsd:element ref="darwin:DecimalLongitude"/>
#				<xsd:element ref="darwin:DecimalLatitude"/>
#				<xsd:element ref="darwin:CoordinatePrecision"/>
#				<xsd:element ref="darwin:BoundingBox"/>
#				<xsd:element ref="darwin:MinimumElevation"/>
#				<xsd:element ref="darwin:MaximumElevation"/>
#				<xsd:element ref="darwin:MinimumDepth"/>
#				<xsd:element ref="darwin:MaximumDepth"/>
#				<xsd:element ref="darwin:Sex"/>
#				<xsd:element ref="darwin:PreparationType"/>
#				<xsd:element ref="darwin:IndividualCount"/>
#				<xsd:element ref="darwin:PreviousCatalogNumber"/>
#				<xsd:element ref="darwin:RelationshipType"/>
#				<xsd:element ref="darwin:RelatedCatalogItem"/>
#				<xsd:element ref="darwin:Notes"/>
#				<xsd:element ref="darwin:ImageURL"/>
#				
#				<xsd:element ref="obis:Source" />
#				<xsd:element ref="obis:Citation" />
#				<xsd:element ref="obis:Subgenus" />
#				<xsd:element ref="obis:StartYearCollected" />
#				<xsd:element ref="obis:EndYearCollected" />
#				<xsd:element ref="obis:StartMonthCollected" />
#				<xsd:element ref="obis:EndMonthCollected" />
#				<xsd:element ref="obis:StartDayCollected" />
#				<xsd:element ref="obis:EndDayCollected" />
#				<xsd:element ref="obis:StartJulianDay" />
#				<xsd:element ref="obis:EndJulianDay" />
#				<xsd:element ref="obis:StartTimeofDay" />
#				<xsd:element ref="obis:EndTimeofDay" />
#				<xsd:element ref="obis:TimeZone" />
#				<xsd:element ref="obis:StartLongitude" />
#				<xsd:element ref="obis:EndLongitude" />
#				<xsd:element ref="obis:StartLatitude" />
#				<xsd:element ref="obis:EndLatitude" />
#				<xsd:element ref="obis:Start_EndCoordinatePrecision" />
#				<xsd:element ref="obis:DepthRange" />
#				<xsd:element ref="obis:Temperature" />
#				<xsd:element ref="obis:LifeStage" />
#				<xsd:element ref="obis:ObservedIndividualCount" />
#				<xsd:element ref="obis:SampleSize" />
#				<xsd:element ref="obis:ObservedWeight" />
#				<xsd:element ref="obis:GMLFeature" />
#			  </xsd:sequence>
#			</xsd:complexType>
#		  </xsd:element>
#		</structure>
#	  </records>
#	  <count>false</count>
#	</search>		
#	</request>''' %(self._type, str(now()), digirResource, destination, filter, start, limit,catalogNumberTag)
		elif self._type == QueryGenerator.NS_TAPIR :
			
			#<externalOutputModel location="http://rs.tdwg.org/tapir/cs/dwc/1.4/model/dw_core_geo_cur.xml"/>
			return '''<?xml version="1.0" encoding="UTF-8"?>
		<request xmlns="http://rs.tdwg.org/tapir/1.0"
         		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         		xsi:schemaLocation="http://rs.tdwg.org/tapir/1.0
               http://rs.tdwg.org/tapir/1.0/schema/tapir.xsd">
			<header>
				<source sendtime="%s"/>
			</header>
			<search count="%i"  start="%i" 	  limit="%i">
			%s
	 		    <filter>
			     %s
			   </filter>
		  </search>
	 </request>'''%(str(now()),count,start,limit,outputmodel,filter)


	# ------------------------------------------------------------------------------------
	def getUnitsProtocol(self, contentSchemaNamespace, serviceURL, listOfTriplet,otherCatalogTag=False ):
		'''
		serviceURL: service access point URL. In case of DiGIR services the resource should be prefixed as RESOURCE@ACCESSPOINT
		contentSchemaNamespace: the namespace of the content/conceptual schema
		In the case of DiGIR resources should be provided in the form of  as the destination string.
		tripleID is a table of triple (unitID,collectionID,institutionID)
		'''
		# if digir service_url parse resource of form RESOURCE@ACCESSPOINT
		if serviceURL is not None:
			if serviceURL.count("@") == 1:
				digirResource, destination = serviceURL.split("@")
			else:
				digirResource = ''
				destination   = serviceURL
		else:
			digirResource = ''
			destination   = ''
		
		if self._type == QueryGenerator.NS_TAPIR:
			if serviceURL[len(serviceURL)-1] !="/":
				serviceURL=serviceURL+"/"
			if destination[len(destination)-1] !="/":
				destination=destination+"/"
			clientCap=ClientCapabilities(destination,serviceURL)
			contentSchemaNamespace=clientCap.getCapabilities()
			outputmodel=clientCap.getOutputmodel(contentSchemaNamespace)
			if contentSchemaNamespace is None:
				raise UnknownSchemaError("no namespace")
			if not QueryGenerator.schema2outputmodel.has_key(contentSchemaNamespace.lower()):
				raise UnknownSchemaError(contentSchemaNamespace)
			conceptIDs = QueryGenerator.schema2outputmodel[contentSchemaNamespace.lower()]		
		else:
			# get concept IDs by hardcoded content schema knowledge
			if not QueryGenerator.schema2conceptIDs.has_key(contentSchemaNamespace.lower()):
				raise UnknownSchemaError(contentSchemaNamespace)
			conceptIDs = QueryGenerator.schema2conceptIDs[contentSchemaNamespace.lower()]

		#tag is different for manis provider
		if otherCatalogTag:
			catalogNumberTag = QueryGenerator.OTHER_CATALOG_TAG
		else:
			catalogNumberTag = conceptIDs[0] 
		
		triplet = listOfTriplet[0]
		unitID =triplet[0]
		collID =triplet[1]
		instID =triplet[2]
		
		# prepare filter string
		filter = self._getOperatorString(conceptID=catalogNumberTag, value=unitID)

		if collID is not None:
			op2 = self._getOperatorString(conceptID=conceptIDs[1], value=collID)
			filter = self._logicallyCombineOperators(filter, op2, 'and')
		if instID is not None:
			op2 = self._getOperatorString(conceptID=conceptIDs[2], value=instID)
			filter = self._logicallyCombineOperators(filter, op2, 'and')
			
		#add fields to filter
		if len(listOfTriplet)>1:
			i=1
			while i < len(listOfTriplet):
				triplet = listOfTriplet[i]
				unitID =triplet[0]
				collID =triplet[1]
				instID =triplet[2]
				
				filter2 = self._getOperatorString(conceptID=catalogNumberTag, value=unitID)

				if collID is not None:
					op2 = self._getOperatorString(conceptID=conceptIDs[1], value=collID)
					filter2 = self._logicallyCombineOperators(filter2, op2, 'and')
				if instID is not None:
					op2 = self._getOperatorString(conceptID=conceptIDs[2], value=instID)
					filter2 = self._logicallyCombineOperators(filter2, op2, 'and') 
					
				filter = self._logicallyCombineOperators(filter, filter2, 'or') 
				i+=1
		
		
		# other things
		start = 0
		limit = 10
		count = False
		# build protocol instance string
		if self._type == QueryGenerator.NS_BioCASe:
			return u'''<?xml version="1.0" encoding="UTF-8"?>
		<!--XML generated by BioCASE query tool for a single database, written by Markus Doering and Javier de la Torre, Botanic Garden and Botanical Museum, Berlin.-->
		<request 
			xmlns="%s" 
			xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
			xsi:schemaLocation="%s http://www.bgbm.org/biodivinf/Schema/protocol_1_3.xsd">
			<header>
				<version software="querytool">%s</version>
				<sendTime>%s</sendTime>
				<source/>
				<destination>%s</destination>
				<type>search</type>
			</header>
			<search>
				<requestFormat>%s</requestFormat>
				<responseFormat start="%i" limit="%i">%s</responseFormat>
				<filter>
				  %s
				</filter>
				<count>%s</count>
			</search>
		</request>
		''' %(self._type, self._type, __version__, str(now()), destination, contentSchemaNamespace, start, limit, contentSchemaNamespace, filter, str(count))
	
		elif self._type == QueryGenerator.NS_DiGIR :
			return  u'''<?xml version="1.0" encoding="UTF-8"?>
		<request xmlns='http://digir.net/schema/protocol/2003/1.0'
	xmlns:xsd='http://www.w3.org/2001/XMLSchema'
	xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
	xmlns:digir='http://digir.net/schema/protocol/2003/1.0'
	xmlns:darwin='http://digir.net/schema/conceptual/darwin/2003/1.0'
	xmlns:dwc='http://digir.net/schema/conceptual/darwin/2003/1.0'
	xsi:schemaLocation='http://digir.net/schema/protocol/2003/1.0 http://digir.sourceforge.net/schema/protocol/2003/1.0/digir.xsd http://digir.net/schema/conceptual/darwin/2003/1.0 http://digir.sourceforge.net/schema/conceptual/darwin/2003/1.0/darwin2.xsd'>
 
        
		<header>
		<version>1.0.0</version>
		<sendTime>%s</sendTime>
		<source/>
		<destination resource="%s">%s</destination>
		<type>search</type>
		</header>
		<search>
		<filter>%s</filter>
		<records  start="%i" limit="%i">
		<structure schemaLocation="http://search.biocase.org/europe/static/darwinObis.xsd"/>
		</records>
		<count>false</count>
		</search>
		</request>'''%( str(now()), digirResource, destination, filter,start, limit)
#			return u'''<?xml version="1.0" encoding="UTF-8"?>
#<request 
#	xmlns="%s" 
#	xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
#	xmlns:digir="http://digir.net/schema/protocol/2003/1.0"
#	xmlns:darwin="http://digir.net/schema/conceptual/darwin/2003/1.0"
#	xmlns:obis="http://www.iobis.org/obis"
#	xmlns:manis="http://digir.net/schema/protocol/2003/1.0"
#	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" >
#	<header>
#	  <version>1.0</version>
#	  <sendTime>%s</sendTime>
#	  <source/>
#	  <destination resource="%s">%s</destination>
#	  <type>search</type>
#	</header>
#	<search>
#	  <filter>
#		  %s
#	  </filter>
#	  <records start="%i" limit="%i">
#		<structure>
#		  <xsd:element name="record">
#			<xsd:complexType>
#			  <xsd:sequence>
#				<xsd:element ref="darwin:DateLastModified"/>
#				<xsd:element ref="darwin:InstitutionCode"/>
#				<xsd:element ref="darwin:CollectionCode"/>
#				<xsd:element ref="darwin:%s"/>
#				<xsd:element ref="darwin:ScientificName"/>
#				<xsd:element ref="darwin:BasisOfRecord"/>
#				<xsd:element ref="darwin:Kingdom"/>
#				<xsd:element ref="darwin:Phylum"/>
#				<xsd:element ref="darwin:Class"/>
#				<xsd:element ref="darwin:Order"/>
#				<xsd:element ref="darwin:Family"/>
#				<xsd:element ref="darwin:Genus"/>
#				<xsd:element ref="darwin:Species"/>
#				<xsd:element ref="darwin:Subspecies"/>
#				<xsd:element ref="darwin:ScientificNameAuthor"/>
#				<xsd:element ref="darwin:IdentifiedBy"/>
#				<xsd:element ref="darwin:YearIdentified"/>
#				<xsd:element ref="darwin:MonthIdentified"/>
#				<xsd:element ref="darwin:DayIdentified"/>
#				<xsd:element ref="darwin:TypeStatus"/>
#				<xsd:element ref="darwin:CollectorNumber"/>
#				<xsd:element ref="darwin:FieldNumber"/>
#				<xsd:element ref="darwin:Collector"/>
#				<xsd:element ref="darwin:YearCollected"/>
#				<xsd:element ref="darwin:MonthCollected"/>
#				<xsd:element ref="darwin:DayCollected"/>
#				<xsd:element ref="darwin:JulianDay"/>
#				<xsd:element ref="darwin:TimeOfDay"/>
#				<xsd:element ref="darwin:ContinentOcean"/>
#				<xsd:element ref="darwin:Country"/>
#				<xsd:element ref="darwin:StateProvince"/>
#				<xsd:element ref="darwin:County"/>
#				<xsd:element ref="darwin:Locality"/>
#				<xsd:element ref="darwin:Longitude"/>
#				<xsd:element ref="darwin:Latitude"/>
#				<xsd:element ref="darwin:DecimalLongitude"/>
#				<xsd:element ref="darwin:DecimalLatitude"/>
#				<xsd:element ref="darwin:CoordinatePrecision"/>
#				<xsd:element ref="darwin:BoundingBox"/>
#				<xsd:element ref="darwin:MinimumElevation"/>
#				<xsd:element ref="darwin:MaximumElevation"/>
#				<xsd:element ref="darwin:MinimumDepth"/>
#				<xsd:element ref="darwin:MaximumDepth"/>
#				<xsd:element ref="darwin:Sex"/>
#				<xsd:element ref="darwin:PreparationType"/>
#				<xsd:element ref="darwin:IndividualCount"/>
#				<xsd:element ref="darwin:PreviousCatalogNumber"/>
#				<xsd:element ref="darwin:RelationshipType"/>
#				<xsd:element ref="darwin:RelatedCatalogItem"/>
#				<xsd:element ref="darwin:Notes"/>
#				<xsd:element ref="darwin:ImageURL"/>
#				
#				<xsd:element ref="obis:Source" />
#				<xsd:element ref="obis:Citation" />
#				<xsd:element ref="obis:Subgenus" />
#				<xsd:element ref="obis:StartYearCollected" />
#				<xsd:element ref="obis:EndYearCollected" />
#				<xsd:element ref="obis:StartMonthCollected" />
#				<xsd:element ref="obis:EndMonthCollected" />
#				<xsd:element ref="obis:StartDayCollected" />
#				<xsd:element ref="obis:EndDayCollected" />
#				<xsd:element ref="obis:StartJulianDay" />
#				<xsd:element ref="obis:EndJulianDay" />
#				<xsd:element ref="obis:StartTimeofDay" />
#				<xsd:element ref="obis:EndTimeofDay" />
#				<xsd:element ref="obis:TimeZone" />
#				<xsd:element ref="obis:StartLongitude" />
#				<xsd:element ref="obis:EndLongitude" />
#				<xsd:element ref="obis:StartLatitude" />
#				<xsd:element ref="obis:EndLatitude" />
#				<xsd:element ref="obis:Start_EndCoordinatePrecision" />
#				<xsd:element ref="obis:DepthRange" />
#				<xsd:element ref="obis:Temperature" />
#				<xsd:element ref="obis:LifeStage" />
#				<xsd:element ref="obis:ObservedIndividualCount" />
#				<xsd:element ref="obis:SampleSize" />
#				<xsd:element ref="obis:ObservedWeight" />
#				<xsd:element ref="obis:GMLFeature" />
#			  </xsd:sequence>
#			</xsd:complexType>
#		  </xsd:element>
#		</structure>
#	  </records>
#	  <count>false</count>
#	</search>		
#	</request>''' %(self._type, str(now()), digirResource, destination, filter, start, limit,catalogNumberTag)
		elif self._type == QueryGenerator.NS_TAPIR :
			return '''<?xml version="1.0" encoding="UTF-8"?>
		<request xmlns="http://rs.tdwg.org/tapir/1.0"
         		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         		xsi:schemaLocation="http://rs.tdwg.org/tapir/1.0
               http://rs.tdwg.org/tapir/1.0/schema/tapir.xsd">
			<header>
				<source sendtime="%s"/>
			</header>
			<search count="%i"  start="%i" 	  limit="%i">
			 %s
	 		    <filter>
			     %s
			   </filter>
		  </search>
	 </request>'''%(str(now()),count,start,limit,outputmodel,filter)

class ClientCapabilities:
	'''A class to send capabilities request to TAPIR providers and get the mapped schema
	'''
	#------------------------------------------------------------------------------------
	def __init__(self,providerURL,accesspoint):
		self.providerURL=providerURL.split('tapirlink/')[0]+"tapirlink/tapir_client.php"
		self.accesspoint=accesspoint
		self.catalogNumber=''
		self.collectionCode=''
		self.institutionCode=''
		self.namespace={}
		
	mappingProps={
		"http://rs.tdwg.org/dwc/dwcore/GlobalUniqueIdentifier":
		'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:GlobalUniqueIdentifier"> <concept id="http://rs.tdwg.org/dwc/dwcore/GlobalUniqueIdentifier" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/DateLastModified":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:DateLastModified"> <concept id="http://rs.tdwg.org/dwc/dwcore/DateLastModified" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/BasisOfRecord":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:BasisOfRecord"> <concept id="http://rs.tdwg.org/dwc/dwcore/BasisOfRecord" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/InstitutionCode":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:InstitutionCode"> <concept id="http://rs.tdwg.org/dwc/dwcore/InstitutionCode" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/CollectionCode":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:CollectionCode"> <concept id="http://rs.tdwg.org/dwc/dwcore/CollectionCode" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/CatalogNumber":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:CatalogNumber"> <concept id="http://rs.tdwg.org/dwc/dwcore/CatalogNumber" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/InformationWithheld":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:InformationWithheld"> <concept id="http://rs.tdwg.org/dwc/dwcore/InformationWithheld"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Remarks":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Remarks"> <concept id="http://rs.tdwg.org/dwc/dwcore/Remarks"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/ScientificName":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:ScientificName"> <concept id="http://rs.tdwg.org/dwc/dwcore/ScientificName" required="true"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/HigherTaxon":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:HigherTaxon"> <concept id="http://rs.tdwg.org/dwc/dwcore/HigherTaxon"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Kingdom":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Kingdom"> <concept id="http://rs.tdwg.org/dwc/dwcore/Kingdom"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Phylum":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Phylum"> <concept id="http://rs.tdwg.org/dwc/dwcore/Phylum"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Class":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Class"> <concept id="http://rs.tdwg.org/dwc/dwcore/Class"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Order":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Order"> <concept id="http://rs.tdwg.org/dwc/dwcore/Order"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Family":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Family"> <concept id="http://rs.tdwg.org/dwc/dwcore/Family"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Genus":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Genus"> <concept id="http://rs.tdwg.org/dwc/dwcore/Genus"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/SpecificEpithet":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:SpecificEpithet"> <concept id="http://rs.tdwg.org/dwc/dwcore/SpecificEpithet"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/InfraspecificRank":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:InfraspecificRank"> <concept id="http://rs.tdwg.org/dwc/dwcore/InfraspecificRank"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/InfraspecificEpithet":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:InfraspecificEpithet"> <concept id="http://rs.tdwg.org/dwc/dwcore/InfraspecificEpithet"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/AuthorYearOfScientificName":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:AuthorYearOfScientificName"> <concept id="http://rs.tdwg.org/dwc/dwcore/AuthorYearOfScientificName"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/NomenclaturalCode":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:NomenclaturalCode"> <concept id="http://rs.tdwg.org/dwc/dwcore/NomenclaturalCode"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/IdentificationQualifier":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:IdentificationQualifier"> <concept id="http://rs.tdwg.org/dwc/dwcore/IdentificationQualifier"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/HigherGeography":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:HigherGeography"> <concept id="http://rs.tdwg.org/dwc/dwcore/HigherGeography"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Continent":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Continent"> <concept id="http://rs.tdwg.org/dwc/dwcore/Continent"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/WaterBody":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:WaterBody"> <concept id="http://rs.tdwg.org/dwc/dwcore/WaterBody"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/IslandGroup":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:IslandGroup"> <concept id="http://rs.tdwg.org/dwc/dwcore/IslandGroup"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Island":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Island"> <concept id="http://rs.tdwg.org/dwc/dwcore/Island"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Country":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Country"> <concept id="http://rs.tdwg.org/dwc/dwcore/Country"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/StateProvince":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:StateProvince"> <concept id="http://rs.tdwg.org/dwc/dwcore/StateProvince"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/County":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:County"> <concept id="http://rs.tdwg.org/dwc/dwcore/County"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Locality":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Locality"> <concept id="http://rs.tdwg.org/dwc/dwcore/Locality"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/MinimumElevationInMeters":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:MinimumElevationInMeters"> <concept id="http://rs.tdwg.org/dwc/dwcore/MinimumElevationInMeters"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/MaximumElevationInMeters":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:MaximumElevationInMeters"> <concept id="http://rs.tdwg.org/dwc/dwcore/MaximumElevationInMeters"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/MinimumDepthInMeters":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:MinimumDepthInMeters"> <concept id="http://rs.tdwg.org/dwc/dwcore/MinimumDepthInMeters"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/MaximumDepthInMeters":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:MaximumDepthInMeters"> <concept id="http://rs.tdwg.org/dwc/dwcore/MaximumDepthInMeters"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/CollectingMethod":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:CollectingMethod"> <concept id="http://rs.tdwg.org/dwc/dwcore/CollectingMethod"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/ValidDistributionFlag":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:ValidDistributionFlag"> <concept id="http://rs.tdwg.org/dwc/dwcore/ValidDistributionFlag"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/EarliestDateCollected":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:EarliestDateCollected"> <concept id="http://rs.tdwg.org/dwc/dwcore/EarliestDateCollected"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/LatestDateCollected":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:LatestDateCollected"> <concept id="http://rs.tdwg.org/dwc/dwcore/LatestDateCollected"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Collector":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Collector"> <concept id="http://rs.tdwg.org/dwc/dwcore/Collector"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Sex":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Sex"> <concept id="http://rs.tdwg.org/dwc/dwcore/Sex"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/LifeStage":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:LifeStage"> <concept id="http://rs.tdwg.org/dwc/dwcore/LifeStage"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/Attributes":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:Attributes"> <concept id="http://rs.tdwg.org/dwc/dwcore/Attributes"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/ImageURL":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:ImageURL"> <concept id="http://rs.tdwg.org/dwc/dwcore/ImageURL"/> </node>',
		"http://rs.tdwg.org/dwc/dwcore/RelatedInformation":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcore:RelatedInformation"> <concept id="http://rs.tdwg.org/dwc/dwcore/RelatedInformation"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/DecimalLatitude":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:DecimalLatitude"> <concept id="http://rs.tdwg.org/dwc/geospatial/DecimalLatitude"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/DecimalLongitude":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:DecimalLongitude"> <concept id="http://rs.tdwg.org/dwc/geospatial/DecimalLongitude"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/GeodeticDatum":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:GeodeticDatum"> <concept id="http://rs.tdwg.org/dwc/geospatial/GeodeticDatum"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/CoordinateUncertaintyInMeters":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:CoordinateUncertaintyInMeters"> <concept id="http://rs.tdwg.org/dwc/geospatial/CoordinateUncertaintyInMeters"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/PointRadiusSpatialFit":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:PointRadiusSpatialFit"> <concept id="http://rs.tdwg.org/dwc/geospatial/PointRadiusSpatialFit"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/FootprintWKT":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:FootprintWKT"> <concept id="http://rs.tdwg.org/dwc/geospatial/FootprintWKT"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/FootprintSpatialFit":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:FootprintSpatialFit"> <concept id="http://rs.tdwg.org/dwc/geospatial/FootprintSpatialFit"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/VerbatimCoordinates":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:VerbatimCoordinates"> <concept id="http://rs.tdwg.org/dwc/geospatial/VerbatimCoordinates"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/VerbatimLatitude":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:VerbatimLatitude"> <concept id="http://rs.tdwg.org/dwc/geospatial/VerbatimLatitude"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/VerbatimLongitude":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:VerbatimLongitude"> <concept id="http://rs.tdwg.org/dwc/geospatial/VerbatimLongitude"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/VerbatimCoordinateSystem":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:VerbatimCoordinateSystem"> <concept id="http://rs.tdwg.org/dwc/geospatial/VerbatimCoordinateSystem"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/GeoreferenceProtocol":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:GeoreferenceProtocol"> <concept id="http://rs.tdwg.org/dwc/geospatial/GeoreferenceProtocol"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/GeoreferenceSources":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:GeoreferenceSources"> <concept id="http://rs.tdwg.org/dwc/geospatial/GeoreferenceSources"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/GeoreferenceVerificationStatus":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:GeoreferenceVerificationStatus"> <concept id="http://rs.tdwg.org/dwc/geospatial/GeoreferenceVerificationStatus"/> </node>',
		"http://rs.tdwg.org/dwc/geospatial/GeoreferenceRemarks":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwgeo:GeoreferenceRemarks"> <concept id="http://rs.tdwg.org/dwc/geospatial/GeoreferenceRemarks"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/CatalogNumberNumeric":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:CatalogNumberNumeric"> <concept id="http://rs.tdwg.org/dwc/curatorial/CatalogNumberNumeric"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/IdentifiedBy":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:IdentifiedBy"> <concept id="http://rs.tdwg.org/dwc/curatorial/IdentifiedBy"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/DateIdentified":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:DateIdentified"> <concept id="http://rs.tdwg.org/dwc/curatorial/DateIdentified"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/CollectorNumber":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:CollectorNumber"> <concept id="http://rs.tdwg.org/dwc/curatorial/CollectorNumber"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/FieldNumber":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:FieldNumber"> <concept id="http://rs.tdwg.org/dwc/curatorial/FieldNumber"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/FieldNotes":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:FieldNotes"> <concept id="http://rs.tdwg.org/dwc/curatorial/FieldNotes"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/VerbatimCollectingDate":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:VerbatimCollectingDate"> <concept id="http://rs.tdwg.org/dwc/curatorial/VerbatimCollectingDate"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/VerbatimElevation":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:VerbatimElevation"> <concept id="http://rs.tdwg.org/dwc/curatorial/VerbatimElevation"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/VerbatimDepth":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:VerbatimDepth"> <concept id="http://rs.tdwg.org/dwc/curatorial/VerbatimDepth"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/Preparations":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:Preparations"> <concept id="http://rs.tdwg.org/dwc/curatorial/Preparations"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/TypeStatus":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:TypeStatus"> <concept id="http://rs.tdwg.org/dwc/curatorial/TypeStatus"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/GenBankNumber":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:GenBankNumber"> <concept id="http://rs.tdwg.org/dwc/curatorial/GenBankNumber"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/OtherCatalogNumbers":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:OtherCatalogNumbers"> <concept id="http://rs.tdwg.org/dwc/curatorial/OtherCatalogNumbers"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/RelatedCatalogedItems":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:RelatedCatalogedItems"> <concept id="http://rs.tdwg.org/dwc/curatorial/RelatedCatalogedItems"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/Disposition":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:Disposition"> <concept id="http://rs.tdwg.org/dwc/curatorial/Disposition"/> </node>',
		"http://rs.tdwg.org/dwc/curatorial/IndividualCount":'<node path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord/dwcur:IndividualCount"> <concept id="http://rs.tdwg.org/dwc/curatorial/IndividualCount"/>'
	}	
	def getCapabilities(self):
		request='''<?xml+version="1.0"+encoding="UTF-8"+?>
				<request+
				++++xmlns="http://rs.tdwg.org/tapir/1.0"
				++++xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
				++++xsi:schemaLocation="http://rs.tdwg.org/tapir/1.0+
				++++++++++++++++++++++++http://rs.tdwg.org/tapir/1.0/schema/tapir.xsd">
				++<header>
				++++<source+sendtime="%s">
				++++++<software+name="tapir_client.php"+version="1.0"/>
				++++</source>
				++</header>
				++<capabilities+/>
				</request>
				'''%str(now())
		params = {'request': request.encode('utf-8'),'local_accesspoint':'%s'%self.accesspoint,'operation':'capabilities','send':'Send Request'}
		params = urllib.urlencode(params)
		try:
			f = urllib.urlopen("%s?%s"%(self.providerURL, params))
			self.response = f.read()			
			try:
				tmp=unicode(self.response, 'utf-8')
				#log.info("FULL WRAPPER RESPONSE %s"%unicode(self.response, 'utf-8'))
			except:
				try:
					self.response = unicode(self.response,"ascii")
				except:
					try:
						self.response = unicode(self.response,"ISO-8859-1")
					except:
						try:
							self.response = unicode(self.response,"utf-16")
						except:
							try:
								self.response = unicode(self.response,"koi8-r")
							except:
								try:
									self.response = unicode(self.response,"ansi")
								except Exception,e:
									log.info("CODING PROBLEM UTF/ISO ansi : %s" %e)
		except Exception,e :
			log.info("HTTP ERROR: WRAPPER CAPABILITIES URL %s COULD NOT BE OPENED" % self.providerURL)
			log.info("Error: %s" %e)
			raise HttpError(self.providerURL)
								
		try:
			self.doc = libxml2.parseDoc(self.response)
		except Exception,e:
			self.doc = None
			log.info("ERROR: NO WELL FORMED XML WRAPPER CAPABILITIES RESPONSE FOUND: %s" %e)
			raise MalformedResponseError(self.providerURL, self.response)
		root = self.doc.getRootElement()
		if root.ns() is not None:
			try:
				self._type = unicode(root.ns()).replace('"','').replace("'",'')
				self._type = self._type[self._type.find('=')+1:]
			except:
				self._type = None
		else:
			log.info("CAPABILITIES  PROTOCOL NS NOT found. ROOT ATTRIBUTES: %s" % root.get_properties())
		for header in root.children:
			if header.type == 'element' and header.name=='header':
				self.headerDOM = header
				break
		# try to get main sections
		elem = header
		f=False
		prem=True
		while elem is not None and not f:
			if elem.type == "element":
				if elem.name == 'capabilities':
					capabilitiesDOM=elem
					f=True
			elem = elem.next
		for schemas in capabilitiesDOM.children:
			mapped=[]
			if schemas.type == "element" and schemas.name == 'schema':
				if schemas.hasProp('namespace'):
					nspace=schemas.prop('namespace')
					namespaceDOM=schemas
					for mapping in namespaceDOM.children:
						if mapping.type == "element" and mapping.name == 'mappedConcept':
							if mapping.hasProp('searchable'):
								if str(mapping.prop('searchable')).lower() == 'true':
									if mapping.hasProp('id'):
										mapped.append(mapping.prop('id'))
				self.namespace[namespaceDOM.prop('namespace')]=mapped
				if prem:
					premsnspace=nspace
					prem=False
		if len(self.namespace.keys())==0:
			return None
		return premsnspace
				
				
	def getOutputmodel(self,namespace):
		output=[]
#		output.append('<outputModel xmlns="http://rs.tdwg.org/tapir/1.0"         			xmlns:xs="http://www.w3.org/2001/XMLSchema"      				xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:dwrec="http://rs.tdwg.org/dwc/dwrecord" 						xmlns:dwcore="http://rs.tdwg.org/dwc/dwcore/"  						xmlns:dwgeo="http://rs.tdwg.org/dwc/geospatial/" 						xmlns:dwcur="http://rs.tdwg.org/dwc/curatorial/" 	     				xsi:schemaLocation="http://rs.tdwg.org/tapir/1.0  http://ww2.biocase.org/svn/tapir/trunk/protocol/tapir.xsd http://www.w3.org/2001/XMLSchema http://www.w3.org/2001/XMLSchema.xsd">')
#		output.append('<structure><schema location="http://rs.tdwg.org/dwc/tdwg_dw_record_tapir.xsd"/></structure><mapping>')
#		if 'http://rs.tdwg.org/dwc/dwcore' in namespace:
#			output.append('<indexingElement path="/dwrec:DarwinRecordSet/dwrec:DarwinRecord"/>')
#		for mapped in self.namespace.get(namespace,[]):
#			output.append(ClientCapabilities.mappingProps.get(mapped,''))
#		output.append('</mapping></outputModel>')
		if 'http://rs.tdwg.org/dwc/dwcore' in namespace:
			output.append('<externalOutputModel location="http://rs.tdwg.org/tapir/cs/dwc/1.4/model/dw_core_geo_cur.xml"/>')
		return " ".join(output)
			

		
	
# ----------------------------------------------------------------------------
class ServiceQueryClient:
	'''A class to send queries to wrappers and do simple work with protocol responses.
	Supports DiGIR and BioCASe'''
	
	# ------------------------------------------------------------------------------------
	def __init__(self, protocol='biocase'):
		#log.info("protocol : %s" %protocol)
		if protocol.lower() == 'biocase':
			protocolNS = QueryGenerator.NS_BioCASe
		elif protocol.lower() == 'digir':
			protocolNS = QueryGenerator.NS_DiGIR			
		elif protocol.lower() == 'tapir':
			protocolNS = QueryGenerator.NS_TAPIR
		else:
			raise "Unknown protocol %s" %protocol
		self._type		  = protocolNS
		self.query		  = None # the last query XML protocol string used
		self.response	   = None # the response XML protocol string
		self.doc			= None # the response as a DOM object
		self.contentDOM	 = None # the content DOM element of the response
		self.diagnosticsDOM = None # the diagnostics DOM element of the response
		self.summaryDOM = None #the summary DOM element of the responde for TAPIR
		self.recStatus	  = recordStatus()
		self.QG			 = QueryGenerator(protocolNS=protocolNS)
		
	# ------------------------------------------------------------------------------------
	def getUnitRecord(self, serviceURL, schema, unitID, collID=None, instID=None,otherCatalogTag=False):
#		log.info("getunitRecord %s %s %s"%(unitID,collID,instID))
		protocolXML = self.QG.getUnitProtocol(contentSchemaNamespace=schema, serviceURL=serviceURL, unitID=unitID, collID=collID, instID=instID,otherCatalogTag=otherCatalogTag )
		log.debug("PROTOCOL GENERATED: \n%s" % protocolXML)
		return self.sendQuery(wrapper_url=serviceURL, protocolString=protocolXML)

	# ------------------------------------------------------------------------------------
	def getUnitsRecord(self, serviceURL, schema, listOfTriplet,otherCatalogTag=False):
		protocolXML = self.QG.getUnitsProtocol(contentSchemaNamespace=schema, serviceURL=serviceURL, listOfTriplet=listOfTriplet,otherCatalogTag=otherCatalogTag )
		log.debug("PROTOCOL GENERATED: \n%s" % protocolXML)
		return self.sendQuery(wrapper_url=serviceURL, protocolString=protocolXML)

	#------------------------------------
	def getStandard(self,schema):
		# get concept IDs by hardcoded content schema knowledge
		if self._type == QueryGenerator.NS_TAPIR:
			if not QueryGenerator.schema2outputmodel.has_key(schema.lower()):
				schema=schema[0:len(schema)-1]
			conceptIDs = QueryGenerator.schema2outputmodel[schema.lower()]
			if conceptIDs is None:
				#log.info("schema cherche: %s | liste des schemas %s"%(schema,QueryGenerator.schema2outputmodel.keys()))
				raise UnknownSchemaError(schema)
		else:
			if not QueryGenerator.schema2conceptIDs.has_key(schema.lower()):
				try:
					schema=schema[0:len(schema)-1]
					conceptIDs = QueryGenerator.schema2outputmodel[schema.lower()]
				except:
					#log.info("schema cherche: %s | liste des schemas %s"%(schema,QueryGenerator.schema2outputmodel.keys()))
					raise UnknownSchemaError(schema)
			conceptIDs = QueryGenerator.schema2conceptIDs[schema.lower()]
		#tag is different for manis provider
		standard =conceptIDs[3]
		return standard 
	
	# ------------------------------------------------------------------------------------
	def sendQuery(self, wrapper_url, protocolString, debuglevel=1):
		'''Sends a query to the wrapper and returns a record status object or None if query failed.'''
		cfgObj = configuration.getConfigObj()
		#default time out
		timeout = int(cfgObj.connection_parameter.providerTimeOut)
		timeoutsocket.setDefaultSocketTimeout(timeout)
		
		
		self.query = protocolString
		# if digir destination parse resource of form RESOURCE@ACCESSPOINT
		if wrapper_url.count("@") == 1:
			digirResource, wrapper_url = wrapper_url.split("@")
			#log.info("DiGIR resource %s, embedded in wrapper url, cut off." % (digirResource))
		#log.info("SEND QUERY TO %s"%wrapper_url)
		#We query with the result XML to the wrapper:
		# @request is used by DiGIR, @query by biocase
		if self._type == QueryGenerator.NS_DiGIR or self._type == QueryGenerator.NS_TAPIR:
			params = {'request': protocolString.encode('utf-8')}
			separator="?"
		else:
			params = {'query': protocolString.encode('utf-8'), 'debug':debuglevel}
			if "dbwrapper.py" not in wrapper_url:
				separator="&"
			else:
				separator="?"
		params = urllib.urlencode(params)
		log.info("Service client request: %s %s" %(wrapper_url,params))
		#
		# risky. use error handling
		
		try:
			f = urllib.urlopen("%s%s%s"%(wrapper_url, separator, params))
			self.response = f.read()			
			try:
				tmp=unicode(self.response, 'utf-8')
#				log.info("FULL WRAPPER RESPONSE %s"%unicode(self.response, 'utf-8'))
			except:
				try:
					self.response = unicode(self.response,"ascii")
				except:
					try:
						self.response = unicode(self.response,"ISO-8859-1")
					except:
						try:
							self.response = unicode(self.response,"utf-16")
						except:
							try:
								self.response = unicode(self.response,"koi8-r")
							except:
								try:
									self.response = unicode(self.response,"ansi")
								except Exception,e:
									log.info("CODING PROBLEM UTF/ISO ansi : %s" %e)
		except Exception,e :
			log.info("HTTP ERROR: WRAPPER URL %s COULD NOT BE OPENED" % wrapper_url)
			log.info("Error: %s" %e)
			raise HttpError(wrapper_url)
		tmp=str(self.response).replace("<dwrec:DarwinRecord>","<DarwinRecord>").replace("</dwrec:DarwinRecord>","</DarwinRecord>").replace("><",">\n<")
		
		self.response=tmp		
		
		# parse the xml into xmllib2 DOM:
		
		try:
			self.doc = libxml2.parseDoc(self.response)
		except Exception,e:
			#log.info("type de la reponse: %s" % type(self.response))
			self.doc = None
			log.info("ERROR: NO WELL FORMED XML WRAPPER RESPONSE FOUND: %s" %e)
			raise MalformedResponseError(wrapper_url, self.response)
		#log.info("ANALYZING RESPONSE DOC")
		root = self.doc.getRootElement()
		#log.info("  DOC ROOT %s"%root.name)
		if root.ns() is not None:
			try:
				self._type = unicode(root.ns()).replace('"','').replace("'",'')
				self._type = self._type[self._type.find('=')+1:]
				#log.info("  PROTOCOL NS found: %s" % self._type)
			except:
				self._type = None
		for header in root.children:
			if header.type == 'element' and header.name=='header':
				#log.info("  Header section found.")
				self.headerDOM = header
				break
		# try to get main sections
		elem = header
		while elem is not None:
			if elem.type == "element":
				if elem.name == 'content':
					#log.info("  Content section found.")
					self.contentDOM = elem
				# try to get diagnostics
				elif elem.name == 'search':
					self.summaryDOM=elem
				elif elem.name == 'diagnostics':
					#log.info("  Diagnostics section found.")
					self.diagnosticsDOM = elem
			elem = elem.next

		# try to get record status (paging info)
		# DiGIR keeps record status in the diagnostics
		if self._type == QueryGenerator.NS_DiGIR:
			hasDiagTotal = False
			isTooShort = False
			tooManyErrors = False
			wrongCatalog = False
			for severity, code, diag in self.getDiagnostics():
				#log.info("code: %s" %code)
				if code == 'MATCH_COUNT':
					hasDiagTotal=True
					self.recStatus.total = int( diag )
					#log.info("	Record Total = %i"%(self.recStatus.total))
				elif code == 'RECORD_COUNT':
					self.recStatus.count = int( diag )
					#log.info("	Record Count = %i"%(self.recStatus.count))
				elif code == "UNSUPPORTED_ACCESSPOINT":
					wrongCatalog=True
				elif code == 'END_OF_RECORDS':
					pass
				elif code == 'TOO_MANY_ERRORS':
					tooManyErrors=True
				elif code == 'QUERY_TERM_TOO_SHORT':
					isTooShort=True
				
			if tooManyErrors and isTooShort:
				raise ProtocolError("too short")
			if tooManyErrors and wrongCatalog:
				raise CatalogError("wrong catalog")
			if not hasDiagTotal:
				self.recStatus.total = self.recStatus.count
		# BioCASE
		elif self._type == QueryGenerator.NS_BioCASe and self.contentDOM is not None:
			self.recStatus.start = int( self.contentDOM.prop('recordStart') )
			#log.info("	Record Start = %i"%(self.recStatus.start))
			self.recStatus.count = int( self.contentDOM.prop('recordCount') )
			#log.info("	Record Count = %i"%(self.recStatus.count))
			self.recStatus.drop = int( self.contentDOM.prop('recordDropped') )
			#log.info("	Record Drop = %i"%(self.recStatus.drop))
			try:
				self.recStatus.total = int( self.contentDOM.prop('totalSearchHits') )
				#log.info("	Record Total = %i"%(self.recStatus.total))
			except:
				self.recStatus.total = None						
				#log.info("	Record Total is not existing")
		elif self._type == QueryGenerator.NS_TAPIR:
			hasDiagTotal = False
			isTooShort = False
			tooManyErrors = False
			wrongCatalog = False
			totalReturned = self.getSummary()
			self.recStatus.total= int( totalReturned )
			self.recStatus.count = int( totalReturned )
				#if code == 'MATCH_COUNT':
					#hasDiagTotal=True
					#self.recStatus.total = int( diag )
					##log.info("	Record Total = %i"%(self.recStatus.total))
				#elif code == 'RECORD_COUNT':
					#self.recStatus.count = int( diag )
					##log.info("	Record Count = %i"%(self.recStatus.count))
				#elif code == "UNSUPPORTED_ACCESSPOINT":
					#wrongCatalog=True
				#elif code == 'END_OF_RECORDS':
					#pass
				#elif code == 'TOO_MANY_ERRORS':
					#tooManyErrors=True
				#elif code == 'QUERY_TERM_TOO_SHORT':
					#isTooShort=True
			#if tooManyErrors and isTooShort:
				#raise ProtocolError("too short")
			#if tooManyErrors and wrongCatalog:
				#raise CatalogError("wrong catalog")
			#if not hasDiagTotal:
				#self.recStatus.total = self.recStatus.count
		return self.recStatus

	# ------------------------------------------------------------------------------------
	def getContent(self, asDoc=True):
		noContent =' no content found '
		domerror = 'problem with DOM '
		resError = 'no result'
		if self.recStatus.total == 0:
			log.info("THE PROVIDER RETURNED 0 HIT!!")
			raise NoResultError(resError)
		if self.contentDOM is None or self.contentDOM.children is None:
			log.info("COULDN'T GET CONTENT DOM NODE.")
			raise XMLError(domerror)
		contentDOM = self.contentDOM.copyNodeList()
		n = contentDOM.children
		while n is not None:
			nextnode = n.next
			if n.type == 'element':
				log.info("  Content payload element %s found." % n.name)
			else:
				log.info("  Non payload element %s (type %s) ignored."%(n.name, n.type))
				n.freeNode()
				n.unlinkNode()
			n = nextnode
		# check how many payload elements content has...
		#log.info("  ContentDOM.lsCountNode() = %i" % contentDOM.lsCountNode())
		if contentDOM.lsCountNode() < 1:
			#log.info("NO CONTENT FOUND.")
			raise XMLError(noContent)

		if contentDOM.lsCountNode() == 1:
			payload = contentDOM.children
		else:
			# more than one child element node of content. create artificial recordset node for this digir style
			payload = contentDOM
			payload.setName('RecordSet')
			payload.setNs( contentDOM.children.ns() )
		# add darwin core NS for digir
		if self._type == QueryGenerator.NS_DiGIR  :
			if "xmlns:darwin" not in str(payload):
				payload.setProp('xmlns:darwin','http://digir.net/schema/conceptual/darwin/2003/1.0')
			if "xmlns:xsd" not in str(payload):
				payload.setProp('xmlns:xsd','http://www.w3.org/2001/XMLSchema')
			if "xmlns:xsi" not in str(payload):
				payload.setProp('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance')
			if "xmlns:obis" not in str(payload):
				payload.setProp('xmlns:obis',"http://www.iobis.org/obis")
			if "xmlns:manis" not in str(payload):
				payload.setProp('xmlns:manis',"http://digir.net/schema/protocol/2003/1.0")
		if self._type == QueryGenerator.NS_TAPIR:		
			#if "xmlns:darwin" not in str(payload):
				#payload.setProp('xmlns:darwin','http://digir.net/schema/conceptual/darwin/2003/1.0')	
			if "xmlns:xsd" not in str(payload):
				payload.setProp('xmlns:xsd','http://www.w3.org/2001/XMLSchema')
			if "xmlns:xsi" not in str(payload):
				payload.setProp('xmlns:xsi','http://www.w3.org/2001/XMLSchema-instance')
			if "xmlns:xs" not in str(payload):
				payload.setProp('xmlns:xs','http://www.w3.org/2001/XMLSchema')
			if "xmlns:rdf" not in str(payload):
				payload.setProp('xmlns:rdf','http://www.w3.org/1999/02/22-rdf-syntax-ns#')
			if "xmlns:dwcore" not in str(payload):
				payload.setProp('xmlns:dwcore','http://rs.tdwg.org/dwc/dwcore/')
			if "xmlns:dwgeo" not in str(payload):
				payload.setProp('xmlns:dwgeo','http://rs.tdwg.org/dwc/geospatial/')
			if "xmlns:dwcur" not in str(payload):
				payload.setProp('xmlns:dwcur','http://rs.tdwg.org/dwc/curatorial/')
			
			#if "xmlns:dwrec" not in str(payload):
				#payload.setProp('xmlns:dwrec','http://rs.tdwg.org/dwc/dwrecord/')
		#log.info("payload: %s"%str(payload))
		  

		# create new doc with these nodes:
		if asDoc:
			doc = libxml2.newDoc("1.0")
			doc.setRootElement(payload)
		else:
			doc = payload
		if doc is None:
			raise XMLError(noContent)
		#log.info("CONTENT FOUND: %s" % unicode(doc))
		return doc

	# ------------------------------------------------------------------------------------
	def getDiagnostics(self):
		'''Return a list of triplet tuples of severity,code,content.'''
		diagnostics = []
		if self.diagnosticsDOM is not None and self.diagnosticsDOM.children is not None:
			for diag in self.diagnosticsDOM.children:
				if diag.type == "element" and diag.name == 'diagnostic':
					if diag.hasProp('severity'):
						severity = diag.prop('severity')
					else:
						severity = None
					if diag.hasProp('code'):
						code = diag.prop('code')
					else:
						code = None
					diagnostics.append( (severity, code, unicode(diag.content, 'UTF-8' )) )
		return diagnostics
					
	def getSummary(self):
		'''Return a list of totalreturned.'''
		summary = None
		if self.summaryDOM is not None and self.summaryDOM.children is not None:
			for sum in self.summaryDOM.children:
				if sum.type == "element" and sum.name == 'summary':
					if sum.hasProp('totalReturned'):
						summary = sum.prop('totalReturned')
					else:
						summary = None
				if sum.type == 'element' and sum.name == 'DarwinRecordSet':
					self.contentDOM=sum
		return summary
					



def escapeXML(x):
	'''Escape basic XML entities <>& .'''
	if type(x) in (StringType, UnicodeType):
		x = x.replace('&','&amp;')
		x = x.replace('>','&gt;')
		x = x.replace('<','&lt;')
	return x
	
def checkUmlaut(x):
	umlaut=['ä','Ä','ü','Ü','ö','Ö','&amp;','&']
	umli=False
	for um in umlaut:
		if um in x:
			x=x.replace(um,'%')
			umli=True
	if umli:
		return x
	else:
		return None
