#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
help write out XML documents

>>> import xmlprinter
>>> import StringIO
>>> fp = StringIO.StringIO()
>>> xp = xmlprinter.xmlprinter(fp) # The fp need only have a write() method
>>> xp.startDocument()
>>> xp.notationDecl("html", "-//W3C//DTD XHTML 1.1//EN", "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd")
>>> xp.startElement('html',
...                 {'xmlns': "http://www.w3.org/1999/xhtml",
...                  'xml:lang': "en-us"})
>>> xp.data("\\n")
>>> xp.startElement('head')
>>> xp.startElement('title')
>>> xp.data("This is the title")
>>> xp.endElement()        # we may omit the element name ('title')
>>> xp.endElement('head')  # or we can include it
>>> xp.data("\\n")
>>> xp.startElement('body')
>>> xp.data("\\n")
>>> xp.startElement('p')
>>> xp.data("This is some information in a paragraph.")
>>> xp.endElement('p')
>>> xp.data("\\n")
>>> xp.emptyElement('hr', {'style': 'color: red'})
>>> xp.data("\\n")
>>> xp.endDocument()       # by default closes remaining tags
>>> print fp.getvalue(),
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-us">
<head><title>This is the title</title></head>
<body>
<p>This is some information in a paragraph.</p>
<hr style="color: red" />
</body></html>

This module does nothing fancy like indenting.

Distributions for this module can be downloaded at
https://sourceforge.net/project/showfiles.php?group_id=60881


Copyright (C) 2002 Frank J. Tobin, ftobin@neverending.org

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation; either version 2.1 of the
License, or (at your option) any later version.

The idea for this module was taken from Perl's XML::Writer.
"""

__version__  = "0.1.0"
__revision__ = "$Id: xmlprinter.py 751 2006-09-18 12:00:30Z markus $"

from types import UnicodeType
import xml.sax.saxutils


# saxutils.XMLGenerator.characters(self, content)


class WellFormedError(Exception):
    pass

    
class xmlprinter(object):
    """We try to ensure a well-formed document, but won't check
    things like the validity of element names.
    Method raise WellFormedError if there are well-formed-ness problems.
    """

    xml_version = '1.0'
    
    __slots__ = ['fp', 'encoding', '_elstack', '_inroot',
                 '_past_procinstr', '_past_doctype', '_past_decl', '_finished','_closedBefore']

    def __init__(self, fp):
        """fp is a file-like object, needing only a write() method"""
        self._finished       = False
        self._past_procinstr = False
        self._past_doctype   = False
        self._past_decl      = False
        self._elstack        = []
        self._inroot         = True
        self._closedBefore   = False
        self.fp              = fp
        self.encoding        = 'UTF-8'


    def startDocument(self, encoding='UTF-8',escapeXML=False):
        """Begin writing out a document, including the XML declaration.
        Currently the encoding header can be changed from the default
        and it will affect how the rest of the document is encoded.
        """
        self.encoding = encoding
        
        if self._past_decl and not escapeXML:
            raise WellFormedError, "past allowed point for XML declaration"
        
        if escapeXML:
            self.fp.write(escape('<?xml version=%s encoding=%s?>\n'
                          % (quoteattr(self.xml_version).encode(self.encoding),
                             quoteattr(encoding).encode(self.encoding) )))
        else:    
            self.fp.write('<?xml version=%s encoding=%s?>\n'
                          % (quoteattr(self.xml_version).encode(self.encoding),
                             quoteattr(encoding).encode(self.encoding) ))
        
        self._past_decl = True


    def notationDecl(self, name, public_id=None, system_id=None):
        """Insert DOCTYPE declaration.
        Can only be added right after document start.
        Optional for a well-formed document.
        At least a public_id or system_id must be specified if called."""
        if self._past_doctype:
            raise WellFormedError, "past allowed point for doctype"
        
        self.fp.write('\n<!DOCTYPE %s' % name)

        if public_id is not None and system_id is None:
            raise TypeError, "must have system_id with public_id"

        if public_id is not None:
            self.fp.write(" PUBLIC %s %s" % (quoteattr(public_id).encode(self.encoding),
                                             quoteattr(system_id).encode(self.encoding) ))
        elif system_id is not None:
            self.fp.write(" SYSTEM %s" % quoteattr(system_id).encode(self.encoding) )

        self.fp.write(">\n")
        self._past_doctype = True

    def processingInstruction(self, name, text=''):
        """Include a processing instruction 'name' with some optional text."""
        if self._past_procinstr:
            raise WellFormedError, "past allowed point for processing instructions"
        self.fp.write('\n<?%s %s?>' % (name.encode(self.encoding), escape(text).encode(self.encoding)) )
        self._past_doctype = True
        self._past_decl    = True

    def stylesheet(self, url, type='xsl'):
        """Include a stylesheet processing instruction similar to this:
        <?xml-stylesheet href="mystyle.css" type="text/css"?>"""
        self.processingInstruction(name='xml-stylesheet', text='href="%s" type="text/%s"'%(url,type))

    def comment(self, text, escapeXML=False):
        """Include a comment into the xml stream."""
        if escapeXML:
            self.fp.write(escape('\n<!-- %s -->'%(text),False).encode(self.encoding) )
        else:    
            self.fp.write('\n<!-- %s -->' % (escape(text).encode(self.encoding)) )

    def startElement(self, name, attrs={},nsPrefix=None, escapeXML=False):
        """Start element 'name' with attributes 'attrs'. (<example>)
            if the escapeXML attribute is set to True then the XML is scaped
        """
        self._closedBefore = False
        self._past_doctype = True
        self._past_decl    = True
        self._past_procinstr= True
        
        if self._finished:
            raise WellFormedError, "attempt to add second root element"

        # We store the temporary data in a variable and then check if it must be scaped or not
        _temp_data = []
        
        if nsPrefix:
            _temp_data.append("\n<%s:%s" %(nsPrefix,name))
        else:
            _temp_data.append("\n<%s"%name)
        
        for attr, val in attrs.items():
            if attr is not None and val is not None:
                _temp_data.append(" %s=%s" % (attr, quoteattr(val).encode(self.encoding) ))

        _temp_data.append(">")
        _data = ''.join(_temp_data)
        
        if escapeXML:
            self.fp.write(escape(_data,False).encode(self.encoding) )
        else:
            self.fp.write(_data.encode(self.encoding))
        
        self._elstack.append(name)
        self._inroot = True

    def data(self, data):
        """Add text 'data'."""
        if not self._inroot:
            raise WellFormedError, "attempt to add data outside of root"
        if data is not None:
            self.fp.write(escape(data).encode( self.encoding ))


    def leafElement(self, name, attrs={}, data=None, escapeXML=False):
        """Add a leaf element if there is any data for it in attributes or data."""
        hasAttrData=False
        for attrData in attrs.values():
            if attrData is not None and attrData <> "":
                hasAttrData=True
                break
        if hasAttrData or data is not None: 
            if not self._inroot:
                raise WellFormedError, "attempt to add element outside of root"
            
            # We store the temporary data in a variable and then check if it must be scaped or not
            _temp_data = []
            
            _temp_data.append("\n<%s" % name)
            for attr, val in attrs.items():
                if attr is not None and val is not None:
                    _temp_data.append(" %s=%s" % (attr, quoteattr(val).encode(self.encoding) ))
            if data is not None:
                _temp_data.append(">")
                #Sorry Markus, I am not going to make use of the data method, I am going to repeart it.
                #HACK!!
                if not self._inroot:
                    raise WellFormedError, "attempt to add data outside of root"
                if data is not None:
                    _temp_data.append(escape(data).encode( self.encoding ))
                #HACK!!
                _temp_data.append("</%s>" % name)
            else:
                _temp_data.append(" />")
            self._closedBefore = True
            
            _temp_data.append(">")
            _data = ''.join(_temp_data)
            if escapeXML:
                self.fp.write(escape(_data,False).encode(self.encoding) )
            else:    
                self.fp.write(_data.encode(self.encoding))
        

    def emptyElement(self, name, attrs={}):
        """Add an empty element (<example />)"""
        if not self._inroot:
            raise WellFormedError, "attempt to add element outside of root"

        self.fp.write("\n<%s" % name)
        for attr, val in attrs.items():
            if attr is not None and val is not None:
                self.fp.write(" %s=%s" % (attr, quoteattr(val).encode(self.encoding) ))
        self.fp.write(" />")
        self._closedBefore = True
        

    def endElement(self, name=None, nsPrefix=None, escapeXML=False):
        """End the element 'name'.
        If 'name' is None, then end the most recently-opened element.
        (</example>).

        If the last element is being closed, then it 
        """
        popel = self._elstack.pop()

        if name is not None and name != popel:
            raise WellFormedError, "ending an unstarted element %s" \
                  % repr(name)

        if name is None:
            name = popel
        if self._closedBefore:
            self.fp.write("\n")
        
        # We store the temporary data in a variable and then check if it must be scaped or not
        _temp_data = []
        
        if nsPrefix:
            _temp_data.append("</%s:%s" %(nsPrefix,name))
        else:
            _temp_data.append("</%s" % name)

        _temp_data.append(">")
        _data = ''.join(_temp_data)
        if escapeXML:
            self.fp.write(escape(_data,False).encode(self.encoding) )
        else:    
            self.fp.write(_data.encode(self.encoding))       
        
        if len(self._elstack) == 0:
            self._inroot = False
            # ensures a newline at the end of a text file
            self.fp.write("\n")
        self._closedBefore = True


    def endDocument(self, autoclose=True):
        """Finish up a document.
        If autoclose is True, then also close any unclosed elements.
        Else, all elements must already be closed.
        """
        if self._finished:
            raise WellFormedError, "attempt to re-end a _finished document"

        if autoclose:
            while len(self._elstack) > 0:
                self.endElement()

        if len(self._elstack) > 0:
            raise WellFormedError, "attempt to re-end a _finished document"
        
        self._finished = True

def escape(data, allowQuotations=True):
    """Escape &, <, and > in a string of data; used for character data."""
    if type(data) is not UnicodeType:
        try:
            data=unicode(data)
        except:
            data = unicode(data, errors='replace')
    data = data.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
    if not allowQuotations:
        data = data.replace("'", "&apos;").replace('"', "&quot;")
    return data

def quoteattr(data):
    """Escape and quote an attribute value."""
    if type(data) is not UnicodeType:
        try:
            data=unicode(data)
        except:
            data = unicode(data, errors='replace')
    data = escape(data)

    # We don't just turn " into &quot;, we'll use single quotes
    # if possible to retain the 'look' better.
    if '"' in data:
        if "'" in data:
            data = '"%s"' % data.replace('"', "&quot;")
        else:
            data = "'%s'" % data
    else:
        data = '"%s"' % data

    return data


class escapedXmlStream(object): 
    """This class takes a file-like object (fp) and overrides its write() method
    so that it xml-escapes every string before the original write method is called."""
    def __init__(self, fp):
        self.fp=fp
    def write(self, x):
        self.fp.write(xml.sax.saxutils.escape(x))
    